import cv2
from sahi.prediction import ObjectPrediction, PredictionResult
from sahi.utils.cv import visualize_object_predictions
from yoloxdetect import YoloxDetector

from torchyolo.modelhub.basemodel import YoloDetectionModel


class YoloxDetectionModel(YoloDetectionModel):
    def load_model(self):
        model = YoloxDetector(self.model_path, config_path=self.config_path, device=self.device, hf_model=False)
        model.torchyolo = True
        model.conf = self.confidence_threshold
        model.iou = self.iou_threshold
        model.save = self.save
        model.show = self.show
        self.model = model

    def predict(self, image, yaml_file=None):
        object_prediction_list = []
        predict_list = self.model.predict(image_path=image, image_size=self.image_size)
        boxes, scores, cls_ids, class_names = predict_list[0], predict_list[1], predict_list[2], predict_list[3]
        for i in range(len(boxes)):
            box = boxes[i]
            category_id = int(cls_ids[i])
            score = scores[i]
            if score < self.confidence_threshold:
                continue
            x0 = int(box[0])
            y0 = int(box[1])
            x1 = int(box[2])
            y1 = int(box[3])
            bbox = [x0, y0, x1, y1]
            category_name = class_names[category_id]

            object_prediction = ObjectPrediction(
                bbox=bbox,
                score=score,
                category_id=category_id,
                category_name=category_name,
            )
            object_prediction_list.append(object_prediction)

        prediction_result = PredictionResult(
            object_prediction_list=object_prediction_list,
            image=image,
        )
        if self.save:
            prediction_result.export_visuals(export_dir=self.save_path, file_name=self.output_file_name)

        if self.show:
            image = cv2.imread(image)
            output_image = visualize_object_predictions(image=image, object_prediction_list=object_prediction_list)
            cv2.imshow("Prediction", output_image["image"])
            cv2.waitKey(0)
            cv2.destroyAllWindows()

        return prediction_result



from typing import Optional

from torchyolo.automodel import AutoDetectionModel


class YoloHub:
    def __init__(
        self,
        model_type: str = "yolov5",
        model_path: str = "yolov5s.pt",
        device: str = "cpu",
        image_size: int = 640,
        config_path: Optional[str] = "configs.yolox.yolox_s",
    ):
        self.model_type = model_type
        self.model_path = model_path
        self.config_path = config_path
        self.device = device
        self.conf = 0.45
        self.iou = 0.45
        self.image_size = image_size
        self.save = True
        self.show = False
        self.model = None

        # Load Model
        self.load_model()

    def load_model(self):
        model = AutoDetectionModel.from_pretrained(
            model_type=self.model_type,
            model_path=self.model_path,
            config_path=self.config_path,
            device=self.device,
            confidence_threshold=self.conf,
            iou_threshold=self.iou,
            image_size=self.image_size,
        )
        model.save = self.save
        model.show = self.show
        self.model = model
        return model

    def prediction(self, image, yaml_file=None):
        return self.model.predict(image, yaml_file=yaml_file)

    def view_model(self):
        try:
            from torchview import draw_graph
        except:
            raise ImportError("Please install torchview: pip install torchview")

        if self.model_type == "yolov5" or self.model_type == "yolov7":
            model_arch = self.model.model

        elif self.model_type == "yolov8" or self.model_type == "yolox":
            model_arch = self.model.model.model

        elif self.model_type == "yolov6":
            model_arch = self.model.model.model.model

        model_graph = draw_graph(
            model_arch,
            input_size=(1, 3, 352, 352),
            expand_nested=True,
            depth=3,
        )
        # model_graph.visual_graph.render(format='svg')
        model_graph.visual_graph.view()


if __name__ == "__main__":
    model = YoloHub(model_type="yolov6", model_path="yolov6n.pt", device="cuda:0", image_size=640)
    image = "data/highway.jpg"
    result = model.view_model()



import cv2
from sahi.prediction import ObjectPrediction, PredictionResult
from sahi.utils.cv import visualize_object_predictions
from ultralytics import YOLO

from torchyolo.modelhub.basemodel import YoloDetectionModel


class Yolov8DetectionModel(YoloDetectionModel):
    def load_model(self):
        model = YOLO(self.model_path)
        model.conf = self.confidence_threshold
        model.iou = self.iou_threshold
        self.model = model

    def predict(self, image, yaml_file=None):
        prediction = self.model(image, imgsz=self.image_size)
        object_prediction_list = []
        for _, image_predictions_in_xyxy_format in enumerate(prediction):
            for pred in image_predictions_in_xyxy_format.cpu().detach().numpy():
                x1, y1, x2, y2 = (
                    int(pred[0]),
                    int(pred[1]),
                    int(pred[2]),
                    int(pred[3]),
                )
                bbox = [x1, y1, x2, y2]
                score = pred[4]
                category_name = self.model.model.names[int(pred[5])]
                category_id = pred[5]
                object_prediction = ObjectPrediction(
                    bbox=bbox,
                    category_id=int(category_id),
                    score=score,
                    category_name=category_name,
                )
                object_prediction_list.append(object_prediction)

        prediction_result = PredictionResult(
            object_prediction_list=object_prediction_list,
            image=image,
        )
        if self.save:
            prediction_result.export_visuals(export_dir=self.save_path, file_name=self.output_file_name)

        if self.show:
            image = cv2.imread(image)
            output_image = visualize_object_predictions(image=image, object_prediction_list=object_prediction_list)
            cv2.imshow("Prediction", output_image["image"])
            cv2.waitKey(0)
            cv2.destroyAllWindows()

        return prediction_result



import os

import yaml
from easydict import EasyDict as edict


class YamlParser(edict):
    """
    This is yaml parser based on EasyDict.
    """

    def __init__(self, cfg_dict=None, config_file=None):
        if cfg_dict is None:
            cfg_dict = {}

        if config_file is not None:
            assert os.path.isfile(config_file)
            with open(config_file, "r") as fo:
                yaml_ = yaml.load(fo.read(), Loader=yaml.FullLoader)
                cfg_dict.update(yaml_)

        super(YamlParser, self).__init__(cfg_dict)

    def merge_from_file(self, config_file):
        with open(config_file, "r") as fo:
            yaml_ = yaml.load(fo.read(), Loader=yaml.FullLoader)
            self.update(yaml_)

    def merge_from_dict(self, config_dict):
        self.update(config_dict)


def get_config(config_file: str = None) -> YamlParser:
    """
    This function is used to load config.
    Args:
        config_file: config file path
    Returns:
        config: config
    """
    config = YamlParser(config_file=config_file)
    config.merge_from_file(config_file)
    return config



from typing import Optional

import numpy as np
import torch


class YoloDetectionModel:
    def __init__(
        self,
        model_path: Optional[str] = None,
        config_path: Optional[str] = None,
        device: Optional[str] = None,
        confidence_threshold: float = 0.3,
        iou_threshold: float = 0.5,
        image_size: int = 640,
    ):
        """
        Init object detection model.
        Args:
            model_path: str
                Path for the instance segmentation model weight
            config_path: str
                Path for the mmdetection instance segmentation model config file
            device: str
                Torch device, "cpu" or "cuda"
            iou_threshold: float
                All predictions with IoU < iou_threshold will be discarded
            confidence_threshold: float
                All predictions with score < confidence_threshold will be discarded
            image_size: int
                Inference input size.
        """
        self.model_path = model_path
        self.config_path = config_path
        self.device = device
        self.iou_threshold = iou_threshold
        self.confidence_threshold = confidence_threshold
        self.image_size = image_size
        self.show = False
        self.save = True
        self.yaml_file = "torchyolo/configs/yolov6/coco.yaml"
        if self.save:
            self.save_path = "output"
            self.output_file_name = "prediction_visual"

        # automatically load model if load_at_init is True
        self.load_model()

        if self.device is None:
            self.device = "cuda" if torch.cuda.is_available() else "cpu"

    def load_model(self):
        """
        This function should be implemented in a way that detection model
        should be initialized and set to self.model.
        (self.model_path, self.config_path, and self.device should be utilized)
        """
        raise NotImplementedError()

    def predict(self, image: np.ndarray, yaml_file: str = None):
        """
        This function should be implemented in a way that detection model
        should be initialized and set to self.model.
        (self.model_path, self.config_path, and self.device should be utilized)
        """
        raise NotImplementedError()
